@@ -26,6 +26,8 @@
 #include <string.h>
 
 #include "pool.h"
+#include "pool_relcache.h"
+#include "pool_session_context.h"
 
 /*
  * Create relation cache
@@ -96,6 +98,7 @@ void *pool_search_relcache(POOL_RELCACHE *relcache, POOL_CONNECTION_POOL *backen
 	char query[1024];
 	POOL_SELECT_RESULT *res = NULL;
 	int index = 0;
+	int local_session_id;
 
 	/* Eliminate double quotes */
 	rel = malloc(strlen(table)+1);
@@ -104,6 +107,14 @@ void *pool_search_relcache(POOL_RELCACHE *relcache, POOL_CONNECTION_POOL *backen
 		pool_error("pool_search_relcache: malloc failed");
 		return NULL;
 	}
+
+	local_session_id = pool_get_local_session_id();
+	if (local_session_id < 0)
+	{
+		pool_error("pool_search_relcache: pool_get_local_session_id failed");
+		return NULL;
+	}
+
 	for(i=0;*table;table++)
 	{
 		if (*table != '"')
@@ -122,7 +133,7 @@ void *pool_search_relcache(POOL_RELCACHE *relcache, POOL_CONNECTION_POOL *backen
 		 */
 		if (relcache->cache_is_session_local)
 		{
-			if (relcache->cache[i].session_id != LocalSessionId)
+			if (relcache->cache[i].session_id != local_session_id)
 				continue;
 		}
 
@@ -140,9 +151,9 @@ void *pool_search_relcache(POOL_RELCACHE *relcache, POOL_CONNECTION_POOL *backen
 	/* Not in cache. Check the system catalog */
 	snprintf(query, sizeof(query), relcache->sql, rel);
 
-	per_node_statement_log(backend, MASTER_NODE_ID, query);
+	per_node_statement_log(backend, REAL_MASTER_NODE_ID, query);
 
-	if (do_query(MASTER(backend), query, &res, MAJOR(backend)) != POOL_CONTINUE)
+	if (do_query(backend->slots[REAL_MASTER_NODE_ID]->con, query, &res, MAJOR(backend)) != POOL_CONTINUE)
 	{
 		pool_error("pool_search_relcache: do_query failed");
 		if (res)
@@ -161,7 +172,7 @@ void *pool_search_relcache(POOL_RELCACHE *relcache, POOL_CONNECTION_POOL *backen
 		 */
 		if (relcache->cache_is_session_local)
 		{
-			if (relcache->cache[i].session_id != LocalSessionId)
+			if (relcache->cache[i].session_id != local_session_id)
 			{
 				index = i;
 				break;
@@ -185,7 +196,7 @@ void *pool_search_relcache(POOL_RELCACHE *relcache, POOL_CONNECTION_POOL *backen
 	strncpy(relcache->cache[index].dbname, dbname, MAX_ITEM_LENGTH);
 	strncpy(relcache->cache[index].relname, rel, MAX_ITEM_LENGTH);
 	relcache->cache[index].refcnt = 1;
-	relcache->cache[index].session_id = LocalSessionId;
+	relcache->cache[index].session_id = local_session_id;
 	free(rel);
 
 	/*
@@ -214,3 +225,26 @@ void *int_unregister_func(void *data)
 	/* Nothing to do since no memory was allocated */
 	return NULL;
 }
+
+void *string_register_func(POOL_SELECT_RESULT *res)
+{
+	char	*ret = NULL;
+	int		 i;
+
+	if (res->numrows == 0)
+		return NULL;
+
+	for (i = 0; i < res->numrows; i++)
+	{
+		ret = strdup(res->data[i]);
+		break;
+	}
+
+	return (void *)ret;
+}
+
+void *string_unregister_func(void *data)
+{
+	free(data);
+	return data;
+}
